cordova.define("cordova-plugin-googlemaps.CordovaGoogleMaps", function(require, exports, module) {
if (!cordova) return;

var cordova_exec = require('cordova/exec');
var execCmd = require('./execCmdQueue');

var argscheck = require('cordova/argscheck'),
  utils = require('cordova/utils'),
  event = require('./event'),
  common = require('./Common'),
  BaseClass = require('./BaseClass'),
  BaseArrayClass = require('./BaseArrayClass');

var Map = require('./Map');
var LatLng = require('./LatLng');
var LatLngBounds = require('./LatLngBounds');
var Location = require('./Location');
var Marker = require('./Marker');
var Circle = require('./Circle');
var Polyline = require('./Polyline');
var Polygon = require('./Polygon');
var TileOverlay = require('./TileOverlay');
var GroundOverlay = require('./GroundOverlay');
var HtmlInfoWindow = require('./HtmlInfoWindow');
var KmlOverlay = require('./KmlOverlay');
var encoding = require('./encoding');
var spherical = require('./spherical');
var poly = require('./poly');
var Geocoder = require('./Geocoder');
var LocationService = require('./LocationService');
var Environment = require('./Environment');
var MapTypeId = require('./MapTypeId');

var MAPS = {};

/*****************************************************************************
 * Prevent background, background-color, background-image properties
 *****************************************************************************/
var navDecorBlocker = document.createElement("style");
navDecorBlocker.setAttribute("type", "text/css");
navDecorBlocker.innerText = [
  "html, body, ._gmaps_cdv_ {",
  "   background-image: url() !important;",
  "   background: rgba(0,0,0,0) url() !important;",
  "   background-color: rgba(0,0,0,0) !important;",
  "}",
  "._gmaps_cdv_ .nav-decor {",
  "   background-color: rgba(0,0,0,0) !important;",
  "   background: rgba(0,0,0,0) !important;",
  "   display:none !important;",
  "}"
].join("");
document.head.appendChild(navDecorBlocker);

/*****************************************************************************
 * Add event lister to all html nodes under the <body> tag.
 *****************************************************************************/
(function() {
  if (!document.body || !document.body.firstChild) {
    common.nextTick(arguments.callee, 25);
    return;
  }

  document.body.style.backgroundColor = "rgba(0,0,0,0)";

  document.head.appendChild(navDecorBlocker);

  /*****************************************************************************
   * Name space
   *****************************************************************************/
  var singletonLocationService = new LocationService(execCmd);
  module.exports = {
    event: event,
    Animation: {
      BOUNCE: 'BOUNCE',
      DROP: 'DROP'
    },

    BaseClass: BaseClass,
    BaseArrayClass: BaseArrayClass,
    Map: {
      updateDomPositions: function (mapDiv, htmlElements, useChildren) {
        var mapId = mapDiv.getAttribute('__pluginMapId');
        var domPositionsForMap = [];

        if (htmlElements) {
          for (var i = 0; i < htmlElements.length; i++) {
            var htmlElement = htmlElements[i];
            var rect = common.getBoundingClientRect(htmlElement);
            domPositionsForMap.push(rect);
          }
        }

        if (useChildren) {
          for (var i = 0; i < mapDiv.children.length; i++) {
            var htmlElement = mapDiv.children[i];
            var rect = common.getBoundingClientRect(htmlElement);
            domPositionsForMap.push(rect);
          }
        }

        // Send updated domPositions to native
        cordova_exec(function() {
          // Success callback
        }, function(error) {
          // Error callback
        }, 'CordovaGoogleMaps', 'updateDomPositions', [mapId, domPositionsForMap]);
      },
      getMap: function(div, mapOptions) {

        // Use the given mapId, but strip all special characters because they 
        var givenMapId = mapOptions && mapOptions.mapId ? mapOptions.mapId.replace(/[^a-zA-Z ]/g, '').toLowerCase() : '';
        var mapId = givenMapId + Math.random();
        div.setAttribute("__pluginMapId", mapId);
        var map = new Map(mapId, execCmd);

        MAPS[mapId] = map;

        // Catch all events for this map instance, then pass to the instance.
        document.addEventListener(mapId, nativeCallback.bind(map));

        map.one('remove', function() {
          document.removeEventListener(mapId, nativeCallback);

          var div = map.getDiv();
          if (!div) {
            div = document.querySelector("[__pluginMapId='" + mapId + "']");
          }
          if (div) {
            div.removeAttribute('__pluginMapId');
          }

          MAPS[mapId].destroy();
          delete MAPS[mapId];
          map = undefined;
        });

        map.getMap(mapId, div, mapOptions);

        // If HTML divs are given, update DOM positions
        if (mapOptions.htmlElements || mapOptions.useChildren) {
          updateDomPositions(div, mapOptions.htmlElements, mapOptions.useChildren);
        }

        return map;
      }
    },
    HtmlInfoWindow: HtmlInfoWindow,
    LatLng: LatLng,
    LatLngBounds: LatLngBounds,
    Marker: Marker,
    MapTypeId: MapTypeId,
    environment: Environment,
    Geocoder: Geocoder,
    LocationService: singletonLocationService,
    geometry: {
      encoding: encoding,
      spherical: spherical,
      poly: poly
    }
  };

}());

cordova.addConstructor(function() {
  if (!window.Cordova) {
    window.Cordova = cordova;
  }
  window.plugin = window.plugin || {};
  window.plugin.google = window.plugin.google || {};
  window.plugin.google.maps = window.plugin.google.maps || module.exports;
  document.addEventListener("deviceready", function() {
    // workaround for issue on android-19: Cannot read property 'maps' of undefined
    if (!window.plugin) {
      console.warn('re-init window.plugin');
      window.plugin = window.plugin || {};
    }
    if (!window.plugin.google) {
      console.warn('re-init window.plugin.google');
      window.plugin.google = window.plugin.google || {};
    }
    if (!window.plugin.google.maps) {
      console.warn('re-init window.plugin.google.maps');
      window.plugin.google.maps = window.plugin.google.maps || module.exports;
    }

    // Check the Google Maps Android API v2 if the device platform is Android.
    if (/Android/i.test(window.navigator.userAgent)) {
      //------------------------------------------------------------------------
      // If Google Maps Android API v2 is not available,
      // display the warning alert.
      //------------------------------------------------------------------------
      cordova.exec(null, function(message) {
        alert(message);
      }, 'Environment', 'isAvailable', ['']);
    }
  }, {
    once: true
  });
});

/*****************************************************************************
 * Private functions
 *****************************************************************************/

function nativeCallback(params) {
  var args = params.args || [];
  args.unshift(params.evtName);
  this[params.callback].apply(this, args);
}
});
